# Postburner
An ActiveRecord layer on top of Backburner for inspecting and auditing the
queue, especially for delayed jobs. It isn't meant to be fast, but safe.

It is meant to be complementary to Backburner. Use Backburner as the default
ActiveJob processor for mailers, active storage, and the like. Use a
`Postburner::Job` for things that you want to track.

Comes with a mountable interface that can be password protected with whatever
authentication you use in your project.

Also meant to be a replacement/upgrade for [Que](https://github.com/que-rb/que).
If you need something faster, but backed with ACID compliance, check out Que.
There are some additional features in Postburner such as retained jobs
after processing, stats, per job logging, etc.

Compared to plain [Backburner](https://github.com/nesquena/backburner),
Postburner adds:
- Database Jobs for inspection, linking, auditing, removal (and deletion)
- Direct access to associated [Beanstalk](https://beanstalkd.github.io/) (via [beaneater](https://github.com/beanstalkd/beaneater))
- Job Statistics (lag, attempts, logs, tracked errors)
- Convenience methods to clear tubes, stats, and connections for Beanstalk.

Otherwise, Postburner tries to be a super simple layer on `Backburner::Queue`
and `ActiveRecord`. Every tool with either of those are avilabel in
`Postburner::Job`s.

## Usage

```ruby
class RunDonation < Postburner::Job
  queue 'critical'
  queue_priority 0 # 0 is highest priority
  queue_max_job_retries 0 # don't retry

  def process(args)
    # do long tasks here
    # also have access to self.args
  end
end

RunDonation.create!(args: {donation_id: 123}).queue!
=> {:status=>"INSERTED", :id=>"1139"}
RunDonation.create!(args: {donation_id: 123}).queue! at: Time.zone.now + 2.days
=> {:status=>"INSERTED", :id=>"1140"}

# `delay` takes priority over `at`
RunDonation.create!(args: {donation_id: 123}).queue! delay: 1.hour
=> {:status=>"INSERTED", :id=>"1141"}
```

## Installation

```ruby
# in Gemfile
gem 'postburner'
```

```bash
$ bundle

# install migration, possible to edit and add attributes or indexes as needed.
$ bundle exec rails g postburner:install
```

Add a `config/initializers/backburner.rb` with option found [here](https://github.com/nesquena/backburner#configuration).

Set `Backburner` for `ActiveJob`
```
# config/application.rb
config.active_job.queue_adapter = :backburner
```

Postburner may later provide an adapter, but we recommend using `Postburner::Job` classes
directyly.

Add jobs to `app/jobs/`. There currently is no generator.
```ruby
# app/jobs/run_donation.rb

class RunDonation < Postburner::Job
  queue 'critical'
  queue_priority 0 # 0 is highest priority
  queue_max_job_retries 0 # don't retry

  def process(args)
    # do long tasks here
    # also have access to self.args
  end
end

## Contributing
Submit a pull request. Follow conventions of the project. Be nice.

### V1 TODO
- Basic tests
- Add Authentication modules for engine mount.

### V1+ TODO
- Install generator
  - Sub to backburner
- Job generator
  - Build file in app/jobs
  - Inherit from Postburner::Job
- Job generator
- Add before/after/around hooks
- Add destroy, and remove actions on show page
- Clear tubes.
- Document how/when to use activerecord hooks
- Document how/when to use backburner hooks
- Document how/when to use postburner hooks
- Add logging with Job.args in backburner logs
- MAYBE - ActiveJob adapter

## License
The gem is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).
